﻿'*********************************************************
'
' Copyright (c) Microsoft. All rights reserved.
'
'*********************************************************

Imports System.Linq
Imports System.Text
Imports System.Threading.Tasks
Imports Windows.ApplicationModel
Imports Windows.Devices.Enumeration
Imports Windows.Devices.HumanInterfaceDevice
Imports Windows.Foundation
Imports Windows.Storage
Imports Windows.UI.Core
Imports Windows.UI.Xaml
Imports SDKTemplate
Imports System

Namespace CustomTemperatureSensor
	''' <summary>
	''' The purpose of this class is to demonstrate what to do to a HidDevice when a specific app event
	''' is raised (app suspension and resume) or when the device is disconnected. In addition to handling
	''' the HidDevice, the app's state should also be saved upon app suspension (will not be demonstrated here).
	''' 
	''' This class will also demonstrate how to handle device watcher events.
	''' 
	''' For simplicity, this class will only allow at most one device to be connected at any given time. In order
	''' to make this class support multiple devices, make this class a non-singleton and create multiple instances
	''' of this class; each instance should watch one connected device.
	''' </summary>
	Public Class EventHandlerForDevice
		''' <summary>
		''' Allows for singleton EventHandlerForDevice
		''' </summary>
'INSTANT VB NOTE: The variable eventHandlerForDevice was renamed since Visual Basic does not allow class members with the same name:
		Private Shared eventHandlerForDevice_Renamed As EventHandlerForDevice

		''' <summary>
		''' Used to synchronize threads to avoid multiple instantiations of eventHandlerForDevice.
		''' </summary>
		Private Shared singletonCreationLock As New Object()

'INSTANT VB NOTE: The variable deviceSelector was renamed since Visual Basic does not allow class members with the same name:
		Private deviceSelector_Renamed As String
		Private deviceWatcher As DeviceWatcher

'INSTANT VB NOTE: The variable deviceInformation was renamed since Visual Basic does not allow class members with the same name:
		Private deviceInformation_Renamed As DeviceInformation
'INSTANT VB NOTE: The variable deviceAccessInformation was renamed since Visual Basic does not allow class members with the same name:
		Private deviceAccessInformation_Renamed As DeviceAccessInformation
'INSTANT VB NOTE: The variable device was renamed since Visual Basic does not allow class members with the same name:
		Private device_Renamed As HidDevice

		Private appSuspendEventHandler As SuspendingEventHandler
		Private appResumeEventHandler As EventHandler(Of Object)

		Private deviceCloseCallback As TypedEventHandler(Of EventHandlerForDevice, DeviceInformation)
		Private deviceConnectedCallback As TypedEventHandler(Of EventHandlerForDevice, OnDeviceConnectedEventArgs)

		Private deviceAddedEventHandler As TypedEventHandler(Of DeviceWatcher, DeviceInformation)
		Private deviceRemovedEventHandler As TypedEventHandler(Of DeviceWatcher, DeviceInformationUpdate)
		Private deviceAccessEventHandler As TypedEventHandler(Of DeviceAccessInformation, DeviceAccessChangedEventArgs)

		Private watcherSuspended As Boolean
		Private watcherStarted As Boolean
'INSTANT VB NOTE: The variable isEnabledAutoReconnect was renamed since Visual Basic does not allow class members with the same name:
		Private isEnabledAutoReconnect_Renamed As Boolean

		' A pointer back to the main page.  This is needed if you want to call methods in MainPage such
		' as NotifyUser()
		Private rootPage As MainPage = MainPage.Current

		''' <summary>
		''' Enforces the singleton pattern so that there is only one object handling app events
		''' as it relates to the UsbDevice because this sample app only supports communicating with one device at a time. 
		'''
		''' An instance of EventHandlerForDevice is globally available because the device needs to persist across scenario pages.
		'''
		''' If there is no instance of EventHandlerForDevice created before this property is called,
		''' an EventHandlerForDevice will be created.
		''' </summary>
		Public Shared ReadOnly Property Current() As EventHandlerForDevice
			Get
				If eventHandlerForDevice_Renamed Is Nothing Then
					SyncLock singletonCreationLock
						If eventHandlerForDevice_Renamed Is Nothing Then
							CreateNewEventHandlerForDevice()
						End If
					End SyncLock
				End If

				Return eventHandlerForDevice_Renamed
			End Get
		End Property

		''' <summary>
		''' Creates a new instance of EventHandlerForDevice, enables auto reconnect, and uses it as the Current instance.
		''' </summary>
		Public Shared Sub CreateNewEventHandlerForDevice()
			eventHandlerForDevice_Renamed = New EventHandlerForDevice()
		End Sub

		Public Property OnDeviceClose() As TypedEventHandler(Of EventHandlerForDevice, DeviceInformation)
			Get
				Return deviceCloseCallback
			End Get

			Set(ByVal value As TypedEventHandler(Of EventHandlerForDevice, DeviceInformation))
				deviceCloseCallback = value
			End Set
		End Property

		Public Property OnDeviceConnected() As TypedEventHandler(Of EventHandlerForDevice, OnDeviceConnectedEventArgs)
			Get
				Return deviceConnectedCallback
			End Get

			Set(ByVal value As TypedEventHandler(Of EventHandlerForDevice, OnDeviceConnectedEventArgs))
				deviceConnectedCallback = value
			End Set
		End Property

		Public ReadOnly Property IsDeviceConnected() As Boolean
			Get
				Return (device_Renamed IsNot Nothing)
			End Get
		End Property

		Public ReadOnly Property Device() As HidDevice
			Get
				Return device_Renamed
			End Get
		End Property

		''' <summary>
		''' This DeviceInformation represents which device is connected or which device will be reconnected when
		''' the device is plugged in again (if IsEnabledAutoReconnect is true);.
		''' </summary>
		Public ReadOnly Property DeviceInformation() As DeviceInformation
			Get
				Return deviceInformation_Renamed
			End Get
		End Property

		''' <summary>
		''' Returns DeviceAccessInformation for the device that is currently connected using this EventHandlerForDevice
		''' object.
		''' </summary>
		Public ReadOnly Property DeviceAccessInformation() As DeviceAccessInformation
			Get
				Return deviceAccessInformation_Renamed
			End Get
		End Property

		''' <summary>
		''' DeviceSelector AQS used to find this device
		''' </summary>
		Public ReadOnly Property DeviceSelector() As String
			Get
				Return deviceSelector_Renamed
			End Get
		End Property

		''' <summary>
		''' True if EventHandlerForDevice will attempt to reconnect to the device once it is plugged into the computer again
		''' </summary>
		Public Property IsEnabledAutoReconnect() As Boolean
			Get
				Return isEnabledAutoReconnect_Renamed
			End Get
			Set(ByVal value As Boolean)
				isEnabledAutoReconnect_Renamed = value
			End Set
		End Property

		''' <summary>
		''' This method opens the device using the WinRT Hid API. After the device is opened, save the device
		''' so that it can be used across scenarios.
		'''
		''' It is important that the FromIdAsync call is made on the UI thread because the consent prompt can only be displayed
		''' on the UI thread.
		''' 
		''' This method is used to reopen the device after the device reconnects to the computer and when the app resumes.
		''' </summary>
		''' <param name="deviceInfo">Device information of the device to be opened</param>
		''' <param name="deviceSelector">The AQS used to find this device</param>
		''' <returns>True if the device was successfully opened, false if the device could not be opened for well known reasons.
		''' An exception may be thrown if the device could not be opened for extraordinary reasons.</returns>
		Public async Function OpenDeviceAsync(ByVal deviceInfo As DeviceInformation, ByVal deviceSelector As String) As Task(Of Boolean)
			' This sample uses FileAccessMode.ReadWrite to open the device because we do not want other apps opening our device and 
			' changing the state of our device. FileAccessMode.Read can be used instead.
			device_Renamed = await HidDevice.FromIdAsync(deviceInfo.Id, FileAccessMode.ReadWrite)

			Dim successfullyOpenedDevice As Boolean = False
			Dim notificationStatus As NotifyType
			Dim notificationMessage As String = Nothing

			' Device could have been blocked by user or the device has already been opened by another app.
			If device_Renamed IsNot Nothing Then
				successfullyOpenedDevice = True

				deviceInformation_Renamed = deviceInfo
				Me.deviceSelector_Renamed = deviceSelector

				notificationStatus = NotifyType.StatusMessage
				notificationMessage = "Device " & deviceInformation_Renamed.Id & " opened"

				If appSuspendEventHandler Is Nothing OrElse appResumeEventHandler Is Nothing Then
					RegisterForAppEvents()
				End If

				' User can block the device after it has been opened in the Settings charm. We can detect this by registering for the 
				' DeviceAccessInformation.AccessChanged event
				If deviceAccessEventHandler Is Nothing Then
					RegisterForDeviceAccessStatusChange()
				End If

				' Create and register device watcher events for the device to be opened unless we're reopening the device
				If deviceWatcher Is Nothing Then
					deviceWatcher = DeviceInformation.CreateWatcher(deviceSelector)

					RegisterForDeviceWatcherEvents()
				End If

				If Not watcherStarted Then
					' Start the device watcher after we made sure that the device is opened.
					StartDeviceWatcher()
				End If
			Else
				successfullyOpenedDevice = False

				notificationStatus = NotifyType.ErrorMessage

                Dim deviceAccessStatus As DeviceAccessStatus = DeviceAccessInformation.CreateFromId(deviceInfo.Id).CurrentStatus

                If deviceAccessStatus = deviceAccessStatus.DeniedByUser Then
                    notificationMessage = "Access to the device was blocked by the user : " & deviceInfo.Id
                ElseIf deviceAccessStatus = deviceAccessStatus.DeniedBySystem Then
                    ' This status is most likely caused by app permissions (did not declare the device in the app's package.appxmanifest)
                    ' This status does not cover the case where the device is already opened by another app.
                    notificationMessage = "Access to the device was blocked by the system : " & deviceInfo.Id
                Else
                    ' Most likely the device is opened by another app, but cannot be sure
                    notificationMessage = "Unknown error, possibly opened by another app : " & deviceInfo.Id
                End If
			End If

			MainPage.Current.NotifyUser(notificationMessage, notificationStatus)

			' Notify registered callback handle that the device has been opened
			If deviceConnectedCallback IsNot Nothing Then
                Dim deviceConnectedEventArgs As OnDeviceConnectedEventArgs = New OnDeviceConnectedEventArgs(successfullyOpenedDevice, deviceInfo)

				deviceConnectedCallback(Me, deviceConnectedEventArgs)
			End If

			Return successfullyOpenedDevice
		End Function

		''' <summary>
		''' Closes the device, stops the device watcher, stops listening for app events, and resets object state to before a device
		''' was ever connected.
		''' </summary>
		Public Sub CloseDevice()
			If IsDeviceConnected Then
				CloseCurrentlyConnectedDevice()
			End If

			If deviceWatcher IsNot Nothing Then
				If watcherStarted Then
					StopDeviceWatcher()

					UnregisterFromDeviceWatcherEvents()
				End If

				deviceWatcher = Nothing
			End If

			If deviceAccessInformation_Renamed IsNot Nothing Then
				UnregisterFromDeviceAccessStatusChange()

				deviceAccessInformation_Renamed = Nothing
			End If

			If appSuspendEventHandler IsNot Nothing OrElse appResumeEventHandler IsNot Nothing Then
				UnregisterFromAppEvents()
			End If

			deviceInformation_Renamed = Nothing
			deviceSelector_Renamed = Nothing

			deviceConnectedCallback= Nothing
			deviceCloseCallback = Nothing

			isEnabledAutoReconnect_Renamed = True
		End Sub

		Private Sub New()
			watcherStarted = False
			watcherSuspended = False
			isEnabledAutoReconnect_Renamed = True
		End Sub

		''' <summary>
		''' This method demonstrates how to close the device properly using the WinRT Hid API.
		'''
		''' When the HidDevice is closing, it will cancel all IO operations that are still pending (not complete).
		''' The close will not wait for any IO completion callbacks to be called, so the close call may complete before any of
		''' the IO completion callbacks are called.
		''' The pending IO operations will still call their respective completion callbacks with either a task 
		''' cancelled error or the operation completed.
		''' </summary>
		Private async Sub CloseCurrentlyConnectedDevice()
			If device_Renamed IsNot Nothing Then
				' Notify callback that we're about to close the device
				If deviceCloseCallback IsNot Nothing Then
					deviceCloseCallback(Me, deviceInformation_Renamed)
				End If

				' This closes the handle to the device
				device_Renamed.Dispose()

				device_Renamed = Nothing

				' Save the deviceInformation.Id in case deviceInformation is set to null when closing the
				' device
                Dim deviceId As String = deviceInformation_Renamed.Id

                Await rootPage.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, New DispatchedHandler(Sub() MainPage.Current.NotifyUser(deviceId & "is closed", NotifyType.StatusMessage)))
			End If
		End Sub

		''' <summary>
		''' Register for app suspension/resume events. See the comments
		''' for the event handlers for more information on what is being done to the device.
		'''
		''' We will also register for when the app exists so that we may close the device handle.
		''' </summary>
		Private Sub RegisterForAppEvents()
            appSuspendEventHandler = New SuspendingEventHandler(AddressOf EventHandlerForDevice.Current.OnAppSuspension)
			appResumeEventHandler = New EventHandler(Of Object)(AddressOf EventHandlerForDevice.Current.OnAppResume)

			' This event is raised when the app is exited and when the app is suspended
            AddHandler App.Current.Suspending, appSuspendEventHandler

			AddHandler App.Current.Resuming, appResumeEventHandler
		End Sub

		Private Sub UnregisterFromAppEvents()
			' This event is raised when the app is exited and when the app is suspended
            RemoveHandler App.Current.Suspending, appSuspendEventHandler
			appSuspendEventHandler = Nothing

			RemoveHandler App.Current.Resuming, appResumeEventHandler
			appResumeEventHandler = Nothing
		End Sub

		''' <summary>
		''' Register for Added and Removed events.
		''' Note that, when disconnecting the device, the device may be closed by the system before the OnDeviceRemoved callback is invoked.
		''' </summary>
		Private Sub RegisterForDeviceWatcherEvents()
			deviceAddedEventHandler = New TypedEventHandler(Of DeviceWatcher, DeviceInformation)(AddressOf Me.OnDeviceAdded)

			deviceRemovedEventHandler = New TypedEventHandler(Of DeviceWatcher, DeviceInformationUpdate)(AddressOf Me.OnDeviceRemoved)

            AddHandler deviceWatcher.Added, deviceAddedEventHandler

            AddHandler deviceWatcher.Removed, deviceRemovedEventHandler
		End Sub

		Private Sub UnregisterFromDeviceWatcherEvents()
            RemoveHandler deviceWatcher.Added, deviceAddedEventHandler
			deviceAddedEventHandler = Nothing

            RemoveHandler deviceWatcher.Removed, deviceRemovedEventHandler
			deviceRemovedEventHandler = Nothing
		End Sub

		''' <summary>
		''' Listen for any changed in device access permission. The user can block access to the device while the device is in use.
		''' If the user blocks access to the device while the device is opened, the device's handle will be closed automatically by
		''' the system; it is still a good idea to close the device explicitly so that resources are cleaned up.
		''' 
		''' Note that by the time the AccessChanged event is raised, the device handle may already be closed by the system.
		''' </summary>
		Private Sub RegisterForDeviceAccessStatusChange()
			deviceAccessInformation_Renamed = DeviceAccessInformation.CreateFromId(deviceInformation_Renamed.Id)
			deviceAccessEventHandler = New TypedEventHandler(Of DeviceAccessInformation, DeviceAccessChangedEventArgs)(AddressOf Me.OnDeviceAccessChanged)
            AddHandler deviceAccessInformation_Renamed.AccessChanged, deviceAccessEventHandler
		End Sub

		Private Sub UnregisterFromDeviceAccessStatusChange()
            RemoveHandler deviceAccessInformation_Renamed.AccessChanged, deviceAccessEventHandler
			deviceAccessEventHandler = Nothing
		End Sub

		Private Sub StartDeviceWatcher()
			watcherStarted = True

			If (deviceWatcher.Status <> DeviceWatcherStatus.Started) AndAlso (deviceWatcher.Status <> DeviceWatcherStatus.EnumerationCompleted) Then
				deviceWatcher.Start()
			End If
		End Sub

		Private Sub StopDeviceWatcher()
			If (deviceWatcher.Status = DeviceWatcherStatus.Started) OrElse (deviceWatcher.Status = DeviceWatcherStatus.EnumerationCompleted) Then
				deviceWatcher.Stop()
			End If

			watcherStarted = False
		End Sub

		''' <summary>
		''' If a HidDevice object has been instantiated (a handle to the device is opened), we must close it before the app 
		''' goes into suspension because the API automatically closes it for us if we don't. When resuming, the API will
		''' not reopen the device automatically, so we need to explicitly open the device in the app (Scenario1_DeviceConnect).
		'''
		''' Since we have to reopen the device ourselves when the app resumes, it is good practice to explicitly call the close
		''' in the app as well (For every open there is a close).
		''' 
		''' We must stop the DeviceWatcher because it will continue to raise events even if
		''' the app is in suspension, which is not desired (drains battery). We resume the device watcher once the app resumes again.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="eventArgs"></param>
		Private Sub OnAppSuspension(ByVal sender As Object, ByVal args As SuspendingEventArgs)
			If watcherStarted Then
				watcherSuspended = True
				StopDeviceWatcher()
			Else
				watcherSuspended = False
			End If

			CloseCurrentlyConnectedDevice()
		End Sub

		''' <summary>
		''' When resume into the application, we should reopen a handle to the Hid device again. This will automatically
		''' happen when we start the device watcher again; the device will be re-enumerated and we will attempt to reopen it
		''' if IsEnabledAutoReconnect property is enabled.
		''' 
		''' See OnAppSuspension for why we are starting the device watcher again
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="arg"></param>
		Private Sub OnAppResume(ByVal sender As Object, ByVal args As Object)
			If watcherSuspended Then
				watcherSuspended = False
				StartDeviceWatcher()
			End If
		End Sub

		''' <summary>
		''' Close the device that is opened so that all pending operations are canceled properly.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="deviceInformationUpdate"></param>
		Private Sub OnDeviceRemoved(ByVal sender As DeviceWatcher, ByVal deviceInformationUpdate As DeviceInformationUpdate)
			If IsDeviceConnected AndAlso (deviceInformationUpdate.Id = deviceInformation_Renamed.Id) Then
				' The main reasons to close the device explicitly is to clean up resources, to properly handle errors,
				' and stop talking to the disconnected device.
				CloseCurrentlyConnectedDevice()
			End If
		End Sub

		''' <summary>
		''' Open the device that the user wanted to open if it hasn't been opened yet and auto reconnect is enabled.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="deviceInfo"></param>
		Private async Sub OnDeviceAdded(ByVal sender As DeviceWatcher, ByVal deviceInfo As DeviceInformation)
			If (deviceInformation_Renamed IsNot Nothing) AndAlso (deviceInfo.Id = deviceInformation_Renamed.Id) AndAlso (Not IsDeviceConnected) AndAlso isEnabledAutoReconnect_Renamed Then
						' Any app specific device intialization should be done here because we don't know the state of the device when it is re-enumerated.
				await rootPage.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, New DispatchedHandler(async Sub() isEnabledAutoReconnect_Renamed = await OpenDeviceAsync(deviceInformation_Renamed, deviceSelector_Renamed)))
			End If
		End Sub

		''' <summary>
		''' Close the device if the device access was denied by anyone (system or the user) and reopen it if permissions are allowed again
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="eventArgs"></param>
		Private async Sub OnDeviceAccessChanged(ByVal sender As DeviceAccessInformation, ByVal eventArgs As DeviceAccessChangedEventArgs)
			If (eventArgs.Status = DeviceAccessStatus.DeniedBySystem) OrElse (eventArgs.Status = DeviceAccessStatus.DeniedByUser) Then
				CloseCurrentlyConnectedDevice()
			ElseIf (eventArgs.Status = DeviceAccessStatus.Allowed) AndAlso (deviceInformation_Renamed IsNot Nothing) AndAlso isEnabledAutoReconnect_Renamed Then
						' Any app specific device intialization should be done here because we don't know the state of the device when it is re-enumerated.
				await rootPage.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, New DispatchedHandler(async Sub() isEnabledAutoReconnect_Renamed = await OpenDeviceAsync(deviceInformation_Renamed, deviceSelector_Renamed)))
			End If
		End Sub
	End Class
End Namespace
