'*********************************************************
'
' Copyright (c) Microsoft. All rights reserved.
'
'*********************************************************

Imports System
Imports System.Collections.Generic
Imports System.IO
Imports System.Collections.ObjectModel
Imports Windows.ApplicationModel
Imports Windows.Devices.Enumeration
Imports Windows.Devices.HumanInterfaceDevice
Imports Windows.Foundation
Imports Windows.UI.Core
Imports Windows.UI.Xaml
Imports Windows.UI.Xaml.Controls
Imports Windows.UI.Xaml.Navigation
Imports SDKTemplate
Imports CustomTemperatureSensor
Imports Windows.Storage

Namespace CustomTemperatureSensor
	''' <summary>
	''' Demonstrates how to connect to a Hid device and respond to device disconnects, app suspending and resuming events.
	''' 
	''' To use this sample with your device:
	''' 1) Include the device in the Package.appxmanifest. For instructions, see Package.appxmanifest documentation.
	''' 2) Create a DeviceWatcher object for your device. See the InitializeDeviceWatcher method in this sample.
	''' </summary>
	Public NotInheritable Partial Class DeviceConnect
		Inherits SDKTemplate.Common.LayoutAwarePage
		Private Const ButtonNameDisconnectFromDevice As String = "Disconnect from device"
		Private Const ButtonNameDisableReconnectToDevice As String = "Do not automatically reconnect to device that was just closed"

		' A pointer back to the main page.  This is needed if you want to call methods in MainPage such
		' as NotifyUser()
		Private rootPage As MainPage = MainPage.Current

		Private appSuspendEventHandler As SuspendingEventHandler
		Private appResumeEventHandler As EventHandler(Of Object)

		Private listOfDevices As ObservableCollection(Of DeviceListEntry)

		Private mapDeviceWatchersToDeviceSelector As Dictionary(Of DeviceWatcher, String)
		Private watchersSuspended As Boolean
		Private watchersStarted As Boolean

		' Has all the devices enumerated by the device watcher?
		Private isAllDevicesEnumerated As Boolean

		Public Sub New()
			Me.InitializeComponent()

			listOfDevices = New ObservableCollection(Of DeviceListEntry)()

			mapDeviceWatchersToDeviceSelector = New Dictionary(Of DeviceWatcher, String)()
			watchersStarted = False
			watchersSuspended = False

			isAllDevicesEnumerated = False
		End Sub

		''' <summary>
		''' Invoked when this page is about to be displayed in a Frame.
		''' 
		''' Create the DeviceWatcher objects when the user navigates to this page so the UI list of devices is populated.
		''' </summary>
		''' <param name="eventArgs">Event data that describes how this page was reached.  The Parameter
		''' property is typically used to configure the page.</param>
		Protected Overrides Sub OnNavigatedTo(ByVal eventArgs As NavigationEventArgs)
			' If we are connected to the device or planning to reconnect, we should disable the list of devices
			' to prevent the user from opening a device without explicitly closing or disabling the auto reconnect
			If EventHandlerForDevice.Current.IsDeviceConnected OrElse (EventHandlerForDevice.Current.IsEnabledAutoReconnect AndAlso EventHandlerForDevice.Current.DeviceInformation IsNot Nothing) Then
				UpdateConnectDisconnectButtonsAndList(False)

				' These notifications will occur if we are waiting to reconnect to device when we start the page
				EventHandlerForDevice.Current.OnDeviceConnected = AddressOf Me.OnDeviceConnected
				EventHandlerForDevice.Current.OnDeviceClose = AddressOf Me.OnDeviceClosing
			Else
				UpdateConnectDisconnectButtonsAndList(True)
			End If

			' Begin watching out for events
			StartHandlingAppEvents()

			' Initialize the desired device watchers so that we can watch for when devices are connected/removed
			InitializeDeviceWatchers()
			StartDeviceWatchers()

			DeviceListSource.Source = listOfDevices
		End Sub

		''' <summary>
		''' Unregister from App events and DeviceWatcher events because this page will be unloaded.
		''' </summary>
		''' <param name="eventArgs"></param>
		Protected Overrides Sub OnNavigatedFrom(ByVal eventArgs As NavigationEventArgs)
			StopDeviceWatchers()
			StopHandlingAppEvents()

			' We no longer care about the device being connected
			EventHandlerForDevice.Current.OnDeviceConnected = Nothing
			EventHandlerForDevice.Current.OnDeviceClose = Nothing
		End Sub

		Private async Sub ConnectToDevice_Click(ByVal sender As Object, ByVal eventArgs As RoutedEventArgs)
			Dim selection = ConnectDevices.SelectedItems
			Dim entry As DeviceListEntry = Nothing

			If selection.Count > 0 Then
				Dim obj = selection(0)
				entry = CType(obj, DeviceListEntry)

				If entry IsNot Nothing Then
					' Create an EventHandlerForDevice to watch for the device we are connecting to
					EventHandlerForDevice.CreateNewEventHandlerForDevice()

					' Get notified when the device was successfully connected to or about to be closed
					EventHandlerForDevice.Current.OnDeviceConnected = AddressOf Me.OnDeviceConnected
					EventHandlerForDevice.Current.OnDeviceClose = AddressOf Me.OnDeviceClosing

					' It is important that the FromIdAsync call is made on the UI thread because the consent prompt can only be displayed
					' on the UI thread. Since this method is invoked by the UI, we are already in the UI thread.
					Dim openSuccess As Boolean = await EventHandlerForDevice.Current.OpenDeviceAsync(entry.DeviceInformation, entry.DeviceSelector)

					' Disable connect button if we connected to the device
					UpdateConnectDisconnectButtonsAndList((Not openSuccess))
				End If
			End If
		End Sub

		Private Sub DisconnectFromDevice_Click(ByVal sender As Object, ByVal eventArgs As RoutedEventArgs)
			Dim selection = ConnectDevices.SelectedItems
			Dim entry As DeviceListEntry = Nothing

			' Prevent auto reconnect because we are voluntarily closing it
			' Re-enable the ConnectDevice list and ConnectToDevice button if the connected/opened device was removed.
			EventHandlerForDevice.Current.IsEnabledAutoReconnect = False

			If selection.Count > 0 Then
				Dim obj = selection(0)
				entry = CType(obj, DeviceListEntry)

				If entry IsNot Nothing Then
					EventHandlerForDevice.Current.CloseDevice()
				End If
			End If

			UpdateConnectDisconnectButtonsAndList(True)
		End Sub

		''' <summary>
		''' Initialize device watchers to watch for the custom sensor.
		''' Several other ways of using a device selector to get the desired devices (the devices are really DeviceInformation objects):
		''' 1) Using only the Usage Page and the Usage Id of the device (enumerates any device with the same usage page and usage id)
		''' 2) Using the Usage Page, Usage Id, Vendor Id, Product Id (Same as above except it also matches the VIDs and PIDs)
		'''
		''' GetDeviceSelector return an AQS string that can be passed directly into DeviceWatcher.createWatcher() or  DeviceInformation.createFromIdAsync(). 
		'''
		''' In this sample, a DeviceWatcher will be used to watch for devices because we can detect surprise device removals.
		''' </summary>
		Private Sub InitializeDeviceWatchers()
			' Custom sensor
			Dim CustomSensorSelector = HidDevice.GetDeviceSelector(CustomSensor.Device.UsagePage, CustomSensor.Device.UsageId, CustomSensor.Device.Vid, CustomSensor.Device.Pid)

			' Create a device watcher to look for instances of the custom sensor
			Dim CustomSensorWatcher = DeviceInformation.CreateWatcher(CustomSensorSelector)

			' Allow the EventHandlerForDevice to handle device watcher events that relates or effects our device (i.e. device removal, addition, app suspension/resume)
			AddDeviceWatcher(CustomSensorWatcher, CustomSensorSelector)
		End Sub

		Private Sub StartHandlingAppEvents()
			appSuspendEventHandler = New SuspendingEventHandler(AddressOf Me.OnAppSuspension)
			appResumeEventHandler = New EventHandler(Of Object)(AddressOf Me.OnAppResume)

			' This event is raised when the app is exited and when the app is suspended
			App.Current.Suspending += appSuspendEventHandler

			AddHandler App.Current.Resuming, appResumeEventHandler
		End Sub

		Private Sub StopHandlingAppEvents()
			' This event is raised when the app is exited and when the app is suspended
			App.Current.Suspending -= appSuspendEventHandler

			RemoveHandler App.Current.Resuming, appResumeEventHandler
		End Sub

		''' <summary>
		''' Registers for Added, Removed, and Enumerated events on the provided deviceWatcher before adding it to an internal list.
		''' </summary>
		''' <param name="deviceWatcher"></param>
		''' <param name="deviceSelector">The AQS used to create the device watcher</param>
		Private Sub AddDeviceWatcher(ByVal deviceWatcher As DeviceWatcher, ByVal deviceSelector As String)
			AddHandler deviceWatcher.Added, AddressOf OnDeviceAdded
			AddHandler deviceWatcher.Removed, AddressOf OnDeviceRemoved
			AddHandler deviceWatcher.EnumerationCompleted, AddressOf OnDeviceEnumerationComplete

			mapDeviceWatchersToDeviceSelector.Add(deviceWatcher, deviceSelector)
		End Sub

		''' <summary>
		''' Starts all device watchers including ones that have been individually stopped.
		''' </summary>
		Private Sub StartDeviceWatchers()
			' Start all device watchers
			watchersStarted = True
			isAllDevicesEnumerated = False

			For Each deviceWatcher As DeviceWatcher In mapDeviceWatchersToDeviceSelector.Keys
				If (deviceWatcher.Status <> DeviceWatcherStatus.Started) AndAlso (deviceWatcher.Status <> DeviceWatcherStatus.EnumerationCompleted) Then
					deviceWatcher.Start()
				End If
			Next deviceWatcher
		End Sub

		''' <summary>
		''' Stops all device watchers.
		''' </summary>
		Private Sub StopDeviceWatchers()
			' Stop all device watchers
			For Each deviceWatcher As DeviceWatcher In mapDeviceWatchersToDeviceSelector.Keys
				If (deviceWatcher.Status = DeviceWatcherStatus.Started) OrElse (deviceWatcher.Status = DeviceWatcherStatus.EnumerationCompleted) Then
					deviceWatcher.Stop()
				End If
			Next deviceWatcher

			' Clear the list of devices so we don't have potentially disconnected devices around
			ClearDeviceEntries()

			watchersStarted = False
		End Sub

		''' <summary>
		''' Creates a DeviceListEntry for a device and adds it to the list of devices in the UI
		''' </summary>
		''' <param name="deviceInformation">DeviceInformation on the device to be added to the list</param>
		''' <param name="deviceSelector">The AQS used to find this device</param>
		Private Sub AddDeviceToList(ByVal deviceInformation As DeviceInformation, ByVal deviceSelector As String)
			' search the device list for a device with a matching interface ID
			Dim match = FindDevice(deviceInformation.Id)

			' Add the device if it's new
			If match Is Nothing Then
				' Create a new element for this device interface, and queue up the query of its
				' device information
				match = New DeviceListEntry(deviceInformation, deviceSelector)

				' Add the new element to the end of the list of devices
				listOfDevices.Add(match)
			End If
		End Sub

		Private Sub RemoveDeviceFromList(ByVal deviceId As String)
			' Removes the device entry from the interal list; therefore the UI
			Dim deviceEntry = FindDevice(deviceId)

			listOfDevices.Remove(deviceEntry)
		End Sub

		Private Sub ClearDeviceEntries()
			listOfDevices.Clear()
		End Sub

		''' <summary>
		''' Searches through the existing list of devices for the first DeviceListEntry that has
		''' the specified device Id.
		''' </summary>
		''' <param name="deviceId">Id of the device that is being searched for</param>
		''' <returns>DeviceListEntry that has the provided Id; else a nullptr</returns>
		Private Function FindDevice(ByVal deviceId As String) As DeviceListEntry
			If deviceId IsNot Nothing Then
				For Each entry As DeviceListEntry In listOfDevices
					If entry.DeviceInformation.Id = deviceId Then
						Return entry
					End If
				Next entry
			End If

			Return Nothing
		End Function

		''' <summary>
		''' We must stop the DeviceWatchers because device watchers will continue to raise events even if
		''' the app is in suspension, which is not desired (drains battery). We resume the device watcher once the app resumes again.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="eventArgs"></param>
		Private Sub OnAppSuspension(ByVal sender As Object, ByVal args As SuspendingEventArgs)
			If watchersStarted Then
				watchersSuspended = True
				StopDeviceWatchers()
			Else
				watchersSuspended = False
			End If
		End Sub

		''' <summary>
		''' See OnAppSuspension for why we are starting the device watchers again
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="args"></param>
		Private Sub OnAppResume(ByVal sender As Object, ByVal args As Object)
			If watchersSuspended Then
				watchersSuspended = False
				StartDeviceWatchers()
			End If
		End Sub

		''' <summary>
		''' We will remove the device from the UI
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="deviceInformationUpdate"></param>
		Private async Sub OnDeviceRemoved(ByVal sender As DeviceWatcher, ByVal deviceInformationUpdate As DeviceInformationUpdate)
			await rootPage.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, New DispatchedHandler(Sub()
				rootPage.NotifyUser(deviceInformationUpdate.Id & " was removed.", NotifyType.StatusMessage)
				RemoveDeviceFromList(deviceInformationUpdate.Id)
			End Sub))
		End Sub

		''' <summary>
		''' This function will add the device to the listOfDevices so that it shows up in the UI
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="deviceInformation"></param>
		Private async Sub OnDeviceAdded(ByVal sender As DeviceWatcher, ByVal deviceInformation As DeviceInformation)
			await rootPage.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, New DispatchedHandler(Sub()
				rootPage.NotifyUser(deviceInformation.Id & " was added.", NotifyType.StatusMessage)
				AddDeviceToList(deviceInformation, mapDeviceWatchersToDeviceSelector(sender))
			End Sub))
		End Sub

		''' <summary>
		''' Notify the UI whether or not we are connected to a device
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="args"></param>
		Private async Sub OnDeviceEnumerationComplete(ByVal sender As DeviceWatcher, ByVal args As Object)
					' If we finished enumerating devices and the device has not been connected yet, the OnDeviceConnected method
					' is responsible for selecting the device in the device list (UI); otherwise, this method does that.
						' We will be reconnecting to a device
			await rootPage.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, New DispatchedHandler(Sub()
				isAllDevicesEnumerated = True
				If EventHandlerForDevice.Current.IsDeviceConnected Then
					SelectDeviceInList(EventHandlerForDevice.Current.DeviceInformation.Id)
					ButtonDisconnectFromDevice.Content = ButtonNameDisconnectFromDevice
					rootPage.NotifyUser("Currently connected to: " & EventHandlerForDevice.Current.DeviceInformation.Id, NotifyType.StatusMessage)
				ElseIf EventHandlerForDevice.Current.IsEnabledAutoReconnect AndAlso EventHandlerForDevice.Current.DeviceInformation IsNot Nothing Then
					ButtonDisconnectFromDevice.Content = ButtonNameDisableReconnectToDevice
					rootPage.NotifyUser("Waiting to reconnect to device: " & EventHandlerForDevice.Current.DeviceInformation.Id, NotifyType.StatusMessage)
				Else
					rootPage.NotifyUser("No devices currently selected", NotifyType.StatusMessage)
				End If
			End Sub))
		End Sub

		''' <summary>
		''' If all the devices have been enumerated, select the device in the list we connected to. Otherwise let the EnumerationComplete event
		''' from the device watcher handle the device selection
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="deviceInformation"></param>
		Private Sub OnDeviceConnected(ByVal sender As EventHandlerForDevice, ByVal deviceInformation As DeviceInformation)
			' Find and select our connected device
			If isAllDevicesEnumerated Then
				SelectDeviceInList(EventHandlerForDevice.Current.DeviceInformation.Id)

				ButtonDisconnectFromDevice.Content = ButtonNameDisconnectFromDevice
			End If

			rootPage.NotifyUser("Currently connected to: " & EventHandlerForDevice.Current.DeviceInformation.Id, NotifyType.StatusMessage)
		End Sub

		''' <summary>
		''' The device was closed. If we will autoreconnect to the device, reflect that in the UI
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="deviceInformation"></param>
		Private async Sub OnDeviceClosing(ByVal sender As EventHandlerForDevice, ByVal deviceInformation As DeviceInformation)
					' We were connected to the device that was unplugged, so change the "Disconnect from device" button
					' to "Do not reconnect to device"
			await rootPage.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, New DispatchedHandler(Sub()
				If ButtonDisconnectFromDevice.IsEnabled AndAlso EventHandlerForDevice.Current.IsEnabledAutoReconnect Then
					ButtonDisconnectFromDevice.Content = ButtonNameDisableReconnectToDevice
				End If
			End Sub))
		End Sub

		''' <summary>
		''' Selects the item in the UI's listbox that corresponds to the provided device id. If there are no
		''' matches, we will deselect anything that is selected.
		''' </summary>
		''' <param name="deviceIdToSelect">The device id of the device to select on the list box</param>
		Private Sub SelectDeviceInList(ByVal deviceIdToSelect As String)
			' Don't select anything by default.
			ConnectDevices.SelectedIndex = -1

			For deviceListIndex As Integer = 0 To listOfDevices.Count - 1
				If listOfDevices(deviceListIndex).DeviceInformation.Id = deviceIdToSelect Then
					ConnectDevices.SelectedIndex = deviceListIndex

					Exit For
				End If
			Next deviceListIndex
		End Sub

		''' <summary>
		''' When ButtonConnectToDevice is disabled, ConnectDevices list will also be disabled.
		''' </summary>
		''' <param name="enableConnectButton">The state of ButtonConnectToDevice</param>
		Private Sub UpdateConnectDisconnectButtonsAndList(ByVal enableConnectButton As Boolean)
			ButtonConnectToDevice.IsEnabled = enableConnectButton
			ButtonDisconnectFromDevice.IsEnabled = Not ButtonConnectToDevice.IsEnabled

			ConnectDevices.IsEnabled = ButtonConnectToDevice.IsEnabled
		End Sub
	End Class
End Namespace
