'*********************************************************
'
' Copyright (c) Microsoft. All rights reserved.
'
'*********************************************************

Imports Windows.Devices.Enumeration
Imports Windows.Devices.HumanInterfaceDevice
Imports Windows.Foundation
Imports Windows.Storage.Streams
Imports Windows.UI.Core
Imports Windows.UI.Xaml
Imports Windows.UI.Xaml.Controls
Imports Windows.UI.Xaml.Navigation
Imports SDKTemplate
Imports System.Collections.Generic
Imports System

Namespace CustomTemperatureSensor
    ''' <summary>
    ''' This scenario demonstrates to how register and utilize input reports
    ''' </summary>
    Partial Public NotInheritable Class InputReportEvents
        Inherits SDKTemplate.Common.LayoutAwarePage
        Private inputReportEventHandler As TypedEventHandler(Of HidDevice, HidInputReportReceivedEventArgs)
        Private isRegisteredForInputReportEvents As Boolean

        ' Device that we registered for events with
        Private registeredDevice As HidDevice

        ' A pointer back to the main page.  This is needed if you want to call methods in MainPage such
        ' as NotifyUser()
        Private rootPage As MainPage = MainPage.Current

        Public Sub New()
            isRegisteredForInputReportEvents = False

            Me.InitializeComponent()
        End Sub

        ''' <summary>
        ''' Invoked when this page is about to be displayed in a Frame.
        '''
        ''' We will enable/disable parts of the UI if the device doesn't support it.
        ''' 
        ''' The custom sensor requires an output report to tell the device to start sending interrupts, so we will start it
        ''' just in case we register for the events. We want this to be transparent because it doesn't demonstrate how 
        ''' to register for events.
        ''' </summary>
        ''' <param name="eventArgs">Event data that describes how this page was reached.  The Parameter
        ''' property is typically used to configure the page.</param>
        Protected Overrides Sub OnNavigatedTo(ByVal eventArgs As NavigationEventArgs)
            ' We will disable the scenario that is not supported by the device.
            ' If no devices are connected, none of the scenarios will be shown and an error will be displayed
            Dim deviceScenarios As New Dictionary(Of DeviceType, UIElement)()
            deviceScenarios.Add(DeviceType.CustomSensor, TemperatureSensorScenario)

            Utilities.SetUpDeviceScenarios(deviceScenarios, DeviceScenarioContainer)

            UpdateRegisterEventButtons()
        End Sub

        ''' <summary>
        ''' Reset the device into a known state by undoing everything we did to the device (unregister interrupts and
        ''' stop the device from creating interrupts)
        ''' </summary>
        ''' <param name="eventArgs"></param>
        Protected Overrides Sub OnNavigatedFrom(ByVal eventArgs As NavigationEventArgs)
            UnregisterFromInputReportEvent()
        End Sub

        Private Async Sub OnDeviceClosing(ByVal sender As EventHandlerForDevice, ByVal deviceInformation As DeviceInformation)
            Await rootPage.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, New DispatchedHandler(Sub() UpdateRegisterEventButtons()))
        End Sub

        Private Sub RegisterInterruptEvent_Click(ByVal sender As Object, ByVal eventArgs As Windows.UI.Xaml.RoutedEventArgs)
            If EventHandlerForDevice.Current.IsDeviceConnected Then
                ' Register for input report events

                RegisterForInputReportEvents()

                UpdateRegisterEventButtons()
            Else
                Utilities.NotifyDeviceNotConnected()
            End If

        End Sub

        Private Sub UnregisterInterruptEvent_Click(ByVal sender As Object, ByVal eventArgs As Windows.UI.Xaml.RoutedEventArgs)
            If EventHandlerForDevice.Current.IsDeviceConnected Then
                UnregisterFromInputReportEvent()

                UpdateRegisterEventButtons()
            Else
                Utilities.NotifyDeviceNotConnected()
            End If
        End Sub


        ''' <summary>
        ''' This callback updates the slider and textbox with the current temperature from the Netduino/LM35 sensor
        '''
        ''' This method is called at the frequency specified by the current report interval. 
        ''' 
        ''' Since all input reports share the same event in HidDevice, the app needs to get the HidInputReport 
        ''' from eventArgs.Report and compare report ids and usages with the desired report.
        ''' </summary>
        ''' <param name="sender">HidDevice that the event is being raised from</param> 
        ''' <param name="eventArgs">Contains the HidInputReport that caused the event to raise</param> 
        Private Async Sub OnInputReportEvent(ByVal sender As HidDevice, ByVal eventArgs As HidInputReportReceivedEventArgs)
            ' Retrieve the sensor data
            Dim inputReport As HidInputReport = eventArgs.Report
            Dim buffer As IBuffer = inputReport.Data
            Dim dr As DataReader = DataReader.FromBuffer(buffer)
            Dim dataLength As UInteger = inputReport.Data.Length
            Dim bytes(CInt(dataLength)) As Byte
            dr.ReadBytes(bytes)

            ' Render the sensor data

            Await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, Sub()
                                                                         CurrentReadingText.TextAlignment = TextAlignment.Center
                                                                         CurrentReadingText.Text = bytes(1).ToString()
                                                                         TemperatureSlider.Value = CInt(bytes(1))
                                                                         rootPage.NotifyUser(bytes(1).ToString() & " degrees Fahrenheit, " & bytes(2).ToString() & " millisecond report-interval", NotifyType.StatusMessage)
                                                                     End Sub)
        End Sub

        ''' <summary>
        ''' Register for the event that is triggered when the device sends a report to us.
        ''' 
        ''' All input reports share the same event in HidDevice, so once we register the event, all HidInputReports (regardless of the report id) will
        ''' raise the event. Read the comment above OnInputReportEvent for more information on how to distinguish input reports.
        '''
        ''' The function also saves the event token so that we can unregister from the event later on.
        ''' </summary>
        Private Sub RegisterForInputReportEvents()
            If Not isRegisteredForInputReportEvents Then
                inputReportEventHandler = New TypedEventHandler(Of HidDevice, HidInputReportReceivedEventArgs)(AddressOf Me.OnInputReportEvent)

                ' Remember which device we are registering the device with, in case there is a device disconnect and reconnect. We want to avoid unregistering
                ' an event handler that is not registered with the device object.
                ' Ideally, one should remove the event token (e.g. assign to null) upon the device removal to avoid using it again.
                registeredDevice = EventHandlerForDevice.Current.Device

                AddHandler registeredDevice.InputReportReceived, inputReportEventHandler

                isRegisteredForInputReportEvents = True
            End If
        End Sub

        ''' <summary>
        ''' Unregisters from the HidDevice's InputReportReceived event that was registered for in the RegisterForInputReportEvents();
        ''' </summary>
        Private Sub UnregisterFromInputReportEvent()
            If isRegisteredForInputReportEvents Then
                ' Don't unregister event if the device was removed and reconnected because the endpoint event no longer contains our event handler
                If registeredDevice Is EventHandlerForDevice.Current.Device Then
                    RemoveHandler registeredDevice.InputReportReceived, inputReportEventHandler
                End If

                registeredDevice = Nothing
                isRegisteredForInputReportEvents = False
            End If
        End Sub


        Private Sub UpdateRegisterEventButtons()
            ButtonRegisterTemperatureDetection.IsEnabled = Not isRegisteredForInputReportEvents
            ButtonUnregisterTemperatureDetection.IsEnabled = isRegisteredForInputReportEvents
        End Sub
    End Class
End Namespace
