﻿'*********************************************************
'
' Copyright (c) Microsoft. All rights reserved.
'
'*********************************************************

Imports Windows.Devices.HumanInterfaceDevice
Imports Windows.UI.Xaml
Imports SDKTemplate
Imports System.Collections.Generic

Namespace CustomTemperatureSensor
    Public Class Utilities
        ''' <summary>
        ''' Displays the compatible scenarios and hides the non-compatible ones.
        ''' If there are no supported devices, the scenarioContainer will be hidden and an error message
        ''' will be displayed.
        ''' </summary>
        ''' <param name="scenarios">The key is the device type that the value, scenario, supports.</param>
        ''' <param name="scenarioContainer">The container that encompasses all the scenarios that are specific to devices</param>
        Public Shared Sub SetUpDeviceScenarios(ByVal scenarios As Dictionary(Of DeviceType, UIElement), ByVal scenarioContainer As UIElement)
            Dim supportedScenario As UIElement = Nothing

            If EventHandlerForDevice.Current.IsDeviceConnected Then
                For Each deviceScenario As KeyValuePair(Of DeviceType, UIElement) In scenarios
                    ' Enable the scenario if it's generic or the device type matches
                    If (deviceScenario.Key = DeviceType.Any) OrElse (deviceScenario.Key = Utilities.GetDeviceType(EventHandlerForDevice.Current.Device)) Then
                        ' Make the scenario visible in case other devices use the same scenario and collapsed it.
                        deviceScenario.Value.Visibility = Visibility.Visible

                        supportedScenario = deviceScenario.Value
                    ElseIf deviceScenario.Value IsNot supportedScenario Then ' Don't hide the scenario if it is supported by the current device and is shared by other devices
                        deviceScenario.Value.Visibility = Visibility.Collapsed
                    End If
                Next deviceScenario
            End If

            If supportedScenario Is Nothing Then
                ' Hide the container so that common elements shared across scenarios are also hidden
                scenarioContainer.Visibility = Visibility.Collapsed

                NotifyDeviceNotConnected()
            End If
        End Sub

        ''' <summary>
        ''' Device type of the device provided device.
        ''' </summary>
        ''' <param name="device"></param>
        ''' <returns>The DeviceType of the device or DeviceType.None if there are no devices connected or is not recognized</returns>
        Public Shared Function GetDeviceType(ByVal device As HidDevice) As DeviceType
            If device IsNot Nothing Then
                If device IsNot Nothing AndAlso device.VendorId = CustomSensor.Device.Vid AndAlso device.ProductId = CustomSensor.Device.Pid Then
                    Return DeviceType.CustomSensor
                End If

                Return DeviceType.None
            End If

            Return DeviceType.None
        End Function

        Public Shared Function IsCustomSensorDevice(ByVal device As HidDevice) As Boolean
            Return (GetDeviceType(device) = DeviceType.CustomSensor)
        End Function

        ''' <summary>
        ''' Prints an error message stating that device is not connected
        ''' </summary>
        Public Shared Sub NotifyDeviceNotConnected()
            MainPage.Current.NotifyUser("Device is not connected, please select a plugged in device to try the scenario again", NotifyType.ErrorMessage)
        End Sub
    End Class
End Namespace
