' Custom USB HID Communication Device - Echo Device
' Copyright (c) Secret Labs LLC. All Rights Reserved.
'  
' Licensed under the Apache 2.0 open source license
' 
' 
' The starting point for this firmware was the UsbHidEchoNetduinoApp sample that is found on the Netduino forum
' see:  http://forums.netduino.com/index.php?/topic/1514-driverless-pc-netduino-communication-using-usb/
'  For license terms see: http://creativecommons.org/licenses/by-sa/3.0/
'
'
Imports Microsoft.SPOT
Imports System.Threading
Imports Microsoft.SPOT.Hardware.UsbClient
Imports Microsoft.SPOT.Hardware
Imports System

Namespace NetduinoSensor
    Friend Structure InputReport
        Public Temperature As Byte ' Temperature in degrees Fahrenheit
        Public Interval As Byte ' report interval (or frequency) in seconds
    End Structure

    Friend Structure OutputReport
        Public Interval As Byte ' report interval (or frequency) in seconds
    End Structure

    Friend Class Sensor
        Protected Const WRITE_ENDPOINT As Integer = 1
        Protected Const READ_ENDPOINT As Integer = 2
        Protected Const MAXPOWER_MILLIAMPS As Integer = 200
        Protected Const HID_DESCRIPTOR_TYPE As Integer = &H21

        Public stream As UsbStream
        Public usbController As UsbController

        Protected started As Boolean

        Protected hidGenericReportDescriptorPayload() As Byte

        Public Sub New()

            hidGenericReportDescriptorPayload = New Byte() {&H6, &H55, &HFF, &H9, &HA5, &HA1, &H1, &H9, &HA7, &H15, &H0, &H25, &H96, &H75, &H8, &H95, &H1, &H81, &H2, &H9, &HA8, &H15, &H4B, &H25, &HFF, &H75, &H8, &H95, &H1, &H81, &H2, &H9, &HA9, &H15, &H4B, &H25, &HFF, &H75, &H8, &H95, &H1, &H91, &H2, &HC0}
            ' Input report (device-transmits)
            ' Output report (device-receives)
        End Sub

        Public Function Open() As Boolean
            Dim succeed As Boolean = True

            started = False

            Dim usbControllers() As UsbController = usbController.GetControllers()

            If usbControllers.Length < 1 Then
                succeed = False
            End If

            If succeed Then
                usbController = usbControllers(0)

                Try

                    succeed = ConfigureHID()

                    If succeed Then
                        succeed = usbController.Start()
                    End If

                    If succeed Then
                        stream = usbController.CreateUsbStream(WRITE_ENDPOINT, READ_ENDPOINT)
                    End If
                Catch e1 As Exception
                    succeed = False
                End Try
            End If

            started = True
            Return succeed
        End Function

        Public Function Close() As Boolean
            Dim succeed As Boolean = True

            If started Then
                succeed = usbController.Stop()

                If succeed Then
                    started = False
                End If
            End If

            Return succeed
        End Function

        Protected Function ConfigureHID() As Boolean
            Dim configuration As New Configuration()
            Dim deviceDescriptor As New Configuration.DeviceDescriptor(&H16C0, &H12, &H100)
            deviceDescriptor.bcdUSB = &H110
            deviceDescriptor.bDeviceClass = 0
            deviceDescriptor.bDeviceSubClass = 0
            deviceDescriptor.bMaxPacketSize0 = 8
            deviceDescriptor.iManufacturer = 1
            deviceDescriptor.iProduct = 2
            deviceDescriptor.iSerialNumber = 0

            Dim descriptorLength As Byte

            If hidGenericReportDescriptorPayload.Length = 0 OrElse hidGenericReportDescriptorPayload.Length <= Byte.MaxValue Then
                descriptorLength = CByte(hidGenericReportDescriptorPayload.Length)
            Else
                Return False
            End If

            Dim hidClassDescriptorPayload() As Byte = {&H1, &H11, &H0, &H1, &H22, descriptorLength, &H0}

            ' HID class descriptor
            Dim hidClassDescriptor As New Configuration.ClassDescriptor(HID_DESCRIPTOR_TYPE, hidClassDescriptorPayload)
            ' write endpoint
            Dim writeEndpoint As New Configuration.Endpoint(WRITE_ENDPOINT, configuration.Endpoint.ATTRIB_Interrupt Or configuration.Endpoint.ATTRIB_Write)
            writeEndpoint.wMaxPacketSize = 16 ' packet size: 16 bytes
            writeEndpoint.bInterval = 10 ' interval: 10ms

            ' read endpoint
            Dim readEndpoint As New Configuration.Endpoint(READ_ENDPOINT, configuration.Endpoint.ATTRIB_Interrupt Or configuration.Endpoint.ATTRIB_Read)
            readEndpoint.wMaxPacketSize = 16 ' packet size: 16 bytes
            readEndpoint.bInterval = 10 ' interval: 10ms

            Dim usbInterface As New Configuration.UsbInterface(0, New Configuration.Endpoint() {writeEndpoint, readEndpoint})
            usbInterface.classDescriptors = New Configuration.ClassDescriptor() {hidClassDescriptor}
            usbInterface.bInterfaceClass = &H3 ' Must be 0x03 for any HID device. hid1_11.pdf pp.7
            usbInterface.bInterfaceSubClass = &H0
            usbInterface.bInterfaceProtocol = &H0 ' No interface protocol is supported. hid1_11.pdf pp. 9

            ' configuration descriptor
            Dim configurationDescriptor As New Configuration.ConfigurationDescriptor(MAXPOWER_MILLIAMPS, New Configuration.UsbInterface() {usbInterface})
            configurationDescriptor.bmAttributes = configuration.ConfigurationDescriptor.ATTRIB_Base Or configuration.ConfigurationDescriptor.ATTRIB_SelfPowered

            Dim hidGenericReportDescriptor As New Configuration.GenericDescriptor(&H81, &H2200, hidGenericReportDescriptorPayload)
            hidGenericReportDescriptor.bRequest = &H6 ' GET_DESCRIPTOR
            hidGenericReportDescriptor.wIndex = &H0 ' INTERFACE 0 (zero)

            Dim manufacturerNameStringDescriptor As New Configuration.StringDescriptor(1, "Microsoft Corporation")
            Dim productNameStringDescriptor As New Configuration.StringDescriptor(2, "Custom Device")
            Dim displayNameStringDescriptor As New Configuration.StringDescriptor(4, "Netduino Temperature Sensor")
            Dim friendlyNameStringDescriptor As New Configuration.StringDescriptor(5, "Netduino Temp Sensor")

            configuration.descriptors = New Configuration.Descriptor() {deviceDescriptor, configurationDescriptor, manufacturerNameStringDescriptor, productNameStringDescriptor, displayNameStringDescriptor, friendlyNameStringDescriptor, hidGenericReportDescriptor}

            usbController.Configuration = configuration

            If usbController.ConfigurationError <> usbController.ConfigError.ConfigOK Then
                Return False
            End If

            Return True
        End Function

        Public Function Update(ByVal iTemperature As Integer, ByVal iInterval As Integer) As Integer
            Dim inputReport As New InputReport()
            Dim Interval As Byte = 0

            inputReport.Temperature = CByte(iTemperature)
            inputReport.Interval = CByte(iInterval)

            SendInputReport(inputReport)

            Interval = GetOutputReport()
            Return CInt(Interval)
        End Function

        Protected Function GetOutputReport() As Byte

            Dim outputReport(0) As Byte
            Dim bytesRead As Integer = 0

            If stream.CanRead Then
                bytesRead = stream.Read(outputReport, 0, 1)
            End If

            If bytesRead > 0 Then
                Return outputReport(0)
            Else
                Return 0
            End If
        End Function

        Protected Sub SendInputReport(ByVal report As InputReport)

            Dim inputReport(1) As Byte

            inputReport(0) = CByte(report.Temperature)
            inputReport(1) = CByte(report.Interval)

            stream.Write(inputReport, 0, 2)
        End Sub

    End Class
End Namespace
